@@ -4,7 +4,8 @@ from collections import OrderedDict
 import numpy as np
 import pandas as pd
 import re
-
+import os
+import pickle
 
 def Embed_Seq_Num(seq,aa_idx,maxlength):
     seq_embed = np.zeros((1, maxlength)).astype('int64')
@@ -154,3 +155,48 @@ def Get_DF_Data(file,type_of_data_cut='Fraction_Response',data_cut = 1.0,aa_colu
             df = df_temp
 
     return df
+
+def supertype_conv(df,keep_non_AB_alleles=False):
+    dir_path = os.path.dirname(os.path.realpath(__file__))
+    df_supertypes = pd.read_csv(os.path.join(dir_path,'Supertype_Data_Dict.csv'))
+    hla_dict = dict(zip(df_supertypes['Allele'], df_supertypes['Supertype_2']))
+
+    temp = df.iloc[:,1:]
+
+    hla_list_ab = []
+    for t in temp.iterrows():
+        t = list(filter(None, list(t[1])))
+        t = [x for x in t if isinstance(x,str)]
+        if not keep_non_AB_alleles:
+            t = np.array([x for x in t if x.startswith('A') or x.startswith('B')])
+        hla_list_ab.append(t)
+
+    hla_list_sup = []
+    for h in hla_list_ab:
+        hla_list_sup.append(np.array([hla_dict[x] if x.startswith('A') or x.startswith('B') else x for x in h]))
+
+    hla_sup = pd.DataFrame()
+    colname = df.columns[0]
+    hla_sup[colname] = df[colname]
+    hla_sup = pd.concat([hla_sup, pd.DataFrame(hla_list_sup)], axis=1)
+    return hla_sup
+
+def save_model_data(self,saver,sess,name,get,iteration=0):
+    saver.save(sess, os.path.join(self.Name, 'models', 'model_' + str(iteration), 'model.ckpt'))
+    with open(os.path.join(self.Name, 'models', 'model_type.pkl'), 'wb') as f:
+        pickle.dump([name, get.name, self.use_alpha, self.use_beta,
+                     self.use_v_beta, self.use_d_beta, self.use_j_beta,
+                     self.use_v_alpha, self.use_j_alpha, self.use_hla, self.use_hla_sup, self.keep_non_AB_alleles,
+                     self.lb_v_beta, self.lb_d_beta, self.lb_j_beta,
+                     self.lb_v_alpha, self.lb_j_alpha, self.lb_hla, self.lb,
+                     self.ind], f)
+
+def load_model_data(self):
+    with open(os.path.join(self.Name, 'models', 'model_type.pkl'), 'rb') as f:
+        model_type, get, self.use_alpha, self.use_beta, \
+        self.use_v_beta, self.use_d_beta, self.use_j_beta, \
+        self.use_v_alpha, self.use_j_alpha, self.use_hla, self.use_hla_sup,self.keep_non_AB_alleles, \
+        self.lb_v_beta, self.lb_d_beta, self.lb_j_beta, \
+        self.lb_v_alpha, self.lb_j_alpha, self.lb_hla, self.lb,\
+            self.ind = pickle.load(f)
+    return model_type,get
